SystemOrganization addCategory: #'Signals-Core'!
SystemOrganization addCategory: #'Signals-Tests'!

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector
	"Mediator: Connect a signaling object with a receiver."

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: nil
		queue: nil
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector pattern: argPattern
	"Shadow or reorder arguments before invoking selector on receiver, e.g.,  #(1 4 3)"

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: argPattern
		queue: nil
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector pattern: argPattern queue: aSharedQueue
	"Shadow or reorder arguments before invoking selector on receiver, e.g.,  #(1 4 3)"

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:17'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector pattern: argPattern queue: aSharedQueue blocking: aBoolean
	"Shadow or reorder arguments before invoking selector on receiver, e.g.,  #(1 4 3)"

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: aBoolean! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:25'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector queue: aSharedQueue
	"Store an incoming signal connection request. This is a direct connection, so the receiver will be called synchonously after the signal was emitted. See #emit."
	
	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: false! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:18'!
connect: aSender signal: aSignal to: aReceiver selector: aSelector queue: aSharedQueue blocking: aBoolean

	aSender
		connectSignal: aSignal
		to: aReceiver
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: aBoolean! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:23'!
connect: aSender signal: aSignal toSelector: aSelector
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: nil
		queue: nil
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:24'!
connect: aSender signal: aSignal toSelector: aSelector pattern: argPattern
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: argPattern
		queue: nil
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:24'!
connect: aSender signal: aSignal toSelector: aSelector pattern: argPattern queue: aSharedQueue
	"Create a queued connection. Receiver will not be called directly but a valuable block with the call will be stored into a queue. The queue has to be processed somewhere else. See #emit."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:20'!
connect: aSender signal: aSignal toSelector: aSelector pattern: argPattern queue: aSharedQueue blocking: aBoolean
	"Create a queued connection. Receiver will not be called directly but a valuable block with the call will be stored into a queue. The queue has to be processed somewhere else. See #emit."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: argPattern
		queue: aSharedQueue
		blocking: aBoolean.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:24'!
connect: aSender signal: aSignal toSelector: aSelector queue: aSharedQueue
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: false.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/3/2010 10:20'!
connect: aSender signal: aSignal toSelector: aSelector queue: aSharedQueue blocking: aBoolean
	"Connect a selector from me with the signal of a sender."

	aSender
		connectSignal: aSignal
		to: self
		selector: aSelector
		pattern: nil
		queue: aSharedQueue
		blocking: aBoolean.! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 9/30/2012 13:58:53.23'!
connectSignal: aSignal to: aReceiver selector: aSelector pattern: argPattern queue: aSharedQueue blocking: aBoolean
	"Private. "

	| forceUiProcess numArgsSender numArgsReceiver fixedPattern |
	"Convert pattern array to custom pattern object."
	fixedPattern := argPattern ifNotNil: [SignalConnectionPattern new mapping: argPattern].
	
	"Check if messages are available."
	(self respondsTo: aSignal) ifFalse: [
		Error signal: ('Signal connection failed!! {1} has no signal named #{2}' format: {self. aSignal})].
	(aReceiver respondsTo: aSelector) ifFalse: [
		Error signal: ('Signal connection failed!! {1} has no selector named #{2}' format: {aReceiver. aSelector})].
	
	"Check if sender sends at least enough arguments if pattern is empty."
	numArgsSender := aSignal count: [:c | c = $:].
	numArgsReceiver := aSelector count: [:c | c = $:].
	(numArgsSender < numArgsReceiver and: [fixedPattern isNil])
		ifTrue: [Error signal: 'Could not connect sender to receiver!! Receiver expects more arguments!!'].
	
	"Check if pattern size is correct."
	fixedPattern ifNotNil: [
		((fixedPattern hasIndices not or: [fixedPattern minIndex >= 0])
			and: [numArgsReceiver == fixedPattern size]
			and: [fixedPattern hasIndices not or: [numArgsSender >= fixedPattern maxIndex]])
			ifFalse: [Error signal: 'Could not connect sender to receiver!! Pattern is invalid!!']].
	
	"Auto-truncate unnecessary arguments if pattern is not given."
	(numArgsSender > numArgsReceiver and: [fixedPattern isNil])
		ifTrue: [fixedPattern := SignalConnectionPattern new
			mapping: ((1 to: numArgsReceiver) collect: [:i | i])].

	"Signals sent in a non-multiprogramming environment should be processed in the UI process no matter from which process they are emitted."
	forceUiProcess := Processor activeProcess == UIManager default uiProcess and: [aSharedQueue isNil].
	
	"Add the connection."
	self signalMutex critical: [
		(self signalConnections includesKey: (SignalKey signal: aSignal sender: self))
			ifFalse: [self signalConnections
				at: (SignalKey signal: aSignal sender: self) "configureFinalize"
				put: WeakIdentityKeyDictionary new].
		
		((self signalConnections
			at: (SignalKey signal: aSignal sender: self))
			at: aReceiver
			ifAbsentPut: [OrderedCollection new])
			add: (SignalConnectionConfiguration new
				selector: aSelector;
				pattern: fixedPattern;
				queue: aSharedQueue;
				isBlocking: aBoolean;
				forceUiProcess: forceUiProcess)].
		
	"Signals-Wrapper extension --- If signal is not a signal, make it to a signal."
	"(Smalltalk includesKey: #SignalMethodWrapper)
		ifTrue: [(Smalltalk at: #SignalMethodWrapper) convertToSignal: aSignal inClass: self class]."! !

!Object methodsFor: '*signals-core' stamp: 'mt 10/8/2012 10:04:01.153'!
createAutoSignalConnections
	"Transform all selectors like #onSenderSignal:arg1:arg2: into corresponding connection create statements and execute them. The sender has to be an instance variable with the same name and has to be initialized yet."
	
	| senderSignalPatterns |
	senderSignalPatterns := self class selectors
		select: [:sel | sel beginsWith: #on]
		thenCollect: [:sel | sel allButFirst: 2].
	
	self class allInstVarNames do: [:var | 
		senderSignalPatterns
			select: [:pattern | pattern withFirstCharacterDownshifted beginsWith: var]
			thenDo: [:pattern |
				| sender signal |
				sender := self instVarNamed: var.
				signal := (pattern allButFirst: var size) withFirstCharacterDownshifted asSymbol.
				(sender respondsTo: signal) ifTrue: [ "enable 'lineEdit' and 'lineEditPassword' inst var names"
					self
						connect: sender
						signal: signal
						toSelector: (#on, pattern) asSymbol]]].! !

!Object methodsFor: '*signals-core' stamp: 'mt 3/28/2011 10:28:49.208'!
disconnectAllSignals
	
	self signalMutex critical: [
		self signalConnectionsRepository
			removeKey: self
			ifAbsent: [^ false]].
		
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 3/28/2011 10:26:41.824'!
disconnectSignal: aSignal
	
	self signalMutex critical: [
		self signalConnections
			removeKey: (SignalKey signal: aSignal sender: self)
			ifAbsent: [^ false]].
		
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 3/28/2011 10:26:51.944'!
disconnectSignal: aSignal from: aReceiver
	
	self signalMutex critical: [
		(self signalConnections includesKey: (SignalKey signal: aSignal sender: self))
			ifFalse: [^ false].
			
		(self signalConnections at: (SignalKey signal: aSignal sender: self))
			removeKey: aReceiver
			ifAbsent: [^ false].
	
		(self signalConnections at: (SignalKey signal: aSignal sender: self))
			ifEmpty: [^ self disconnectSignal: aSignal]].
		
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'mt 3/28/2011 10:26:59.702'!
disconnectSignal: aSignal from: aReceiver selector: aSelector
	
	self signalMutex critical: [
		(self signalConnections includesKey: (SignalKey signal: aSignal sender: self))
			ifFalse: [^ false].
			
		((self signalConnections at: (SignalKey signal: aSignal sender: self))
			at: aReceiver
			ifAbsent: [^ false]) "weak"
			removeAllSuchThat: [:connectionConfiguration | connectionConfiguration selector = aSelector].
	
		((self signalConnections at: (SignalKey signal: aSignal sender: self))
			at: aReceiver
			ifAbsent: [^ true]) "weak"
			ifEmpty: [self disconnectSignal: aSignal from: aReceiver]].
	
	^ true! !

!Object methodsFor: '*signals-core' stamp: 'bs 3/14/2011 10:53:53.816'!
emit
	"Calling this from a method makes the method to a signal."

	| signalCtx signal args |
	signalCtx := thisContext sender.
	signal := signalCtx method.
	
	"Do not allow external emitting of signals. Support for method wrappers."
	((signalCtx sender sender receiver isKindOf: (Smalltalk at: #MwMethodWrapper ifAbsent: [nil]))
		ifTrue: [signalCtx sender tempAt: 1]
		ifFalse: [signalCtx sender receiver]) == signalCtx receiver
		ifFalse: [Error signal: 'Must not emit an object''s signal from the outside!!'].
	
	"Extract arguments."
	args := Array new: signal numArgs.
	(1 to: signal numArgs) do: [:index |
		args at: index put: (signalCtx tempAt: index)].
	
	self emitSignal: signal selector arguments: args sender: self.! !

!Object methodsFor: '*signals-core' stamp: 'mt 6/14/2010 19:59:04.461'!
emitAlways
	"Less restrictive version of #emit, e.g., does not check if sender and receiver are the same object. Makes signals emittable from the outside."
	
	| signalCtx signal args |
	signalCtx := thisContext sender.
	signal := signalCtx method.
	
	"Extract arguments."
	args := Array new: signal numArgs.
	(1 to: signal numArgs) do: [:index |
		args at: index put: (signalCtx tempAt: index)].
	
	self emitSignal: signal selector arguments: args sender: self.! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 9/30/2012 13:59:12.869'!
emitSignal: signal arguments: args sender: aSender
	"Private. Performs the triggering of callbacks."

	| actions |
	
	"Fetch targets and schedule dispatch."
	actions := OrderedCollection new. "Buffer actions to free the mutex in time."
	self signalMutex critical: [
		aSender hasSignalConnections ifFalse: [^ self].
		
		(aSender signalConnections at: (SignalKey signal: signal sender: aSender) ifAbsent: [^ self]) associationsDo: [ :association |
			association key ifNotNil: [ :receiver | "Don't let the key go away -> weak dictionary"
				association value do: [ :connectionConfiguration |
					| selector pattern queue filteredArgs blocking forceUiProcess |
					selector := connectionConfiguration selector.
					pattern := connectionConfiguration pattern.
					queue := connectionConfiguration queue.
					blocking := connectionConfiguration isBlocking.
					forceUiProcess := connectionConfiguration forceUiProcess.
					
					forceUiProcess ifTrue: [
						blocking := false. "Never block here."
						queue := Processor activeProcess == UIManager default uiProcess
							ifTrue: [nil] "synchron processing in UI process"
							ifFalse: [WorldState deferredUIMessages]]. "asynchron processing in UI process"
					
					pattern
						ifNil: [ "no argument changes"
							filteredArgs := args]
						ifNotNil: [ "shadowing and reordering"
							filteredArgs := pattern mapArguments: args useSender: aSender].
					actions add: (queue "queued connection or direct connection?"
						ifNil: [[receiver perform: selector withArguments: filteredArgs]]
						ifNotNil: [
							blocking
								ifTrue: [ | thisProcess |
									thisProcess := Processor activeProcess.
									[queue nextPut: [receiver perform: selector withArguments: filteredArgs. thisProcess resume].  thisProcess suspend]]
								ifFalse: [ [queue nextPut: [receiver perform: selector withArguments: filteredArgs]] ] ] ) ] ] ] ].
	
	"Dispatch outside the mutex."
	actions do: [:action | action value].
	
	"TODO: Handle dispatching errors with automatic disconnections."! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 3/28/2011 10:10:37.999'!
hasSignalConnections

	^ self signalConnectionsRepository
		includesKey: self! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 3/28/2011 10:10:10.872'!
signalConnections

	^ self signalConnectionsRepository
		at: self
		ifAbsentPut: [Dictionary new]! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 3/28/2011 10:09:58.428'!
signalConnectionsRepository
	"Return the repository that is used for signal connections for this object."
	
	^ SignalConnectionsRepository instance! !

!Object methodsFor: '*signals-core-private' stamp: 'mt 3/28/2011 10:10:27.535'!
signalMutex

	^ self signalConnectionsRepository mutex! !

Object subclass: #SignalConnectionConfiguration
	instanceVariableNames: 'selector pattern queue isBlocking forceUiProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalConnectionConfiguration commentStamp: 'mt 6/14/2010 19:16:09.16' prior: 0!
A data class that stores connection configuration.!

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:06:43.409'!
forceUiProcess
	"Process callback blocking if already in UI process, enqueue in WorldState>>deferredUIMessages in the other case."

	^ forceUiProcess! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:06:52.015'!
forceUiProcess: aBoolean

	forceUiProcess := aBoolean.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:12:50.2'!
isBlocking
	"Does a queued connection wait until the responsible process performed the callback?"

	^ isBlocking! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:13:39.501'!
isBlocking: aBoolean

	isBlocking := aBoolean.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:14.619'!
pattern
	"Choose and reorder arguments, e.g., #(2 1 3) or access sender with: #(0)."

	^ pattern! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:28.736'!
pattern: anArrayWithIntegers

	pattern := anArrayWithIntegers.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:46.089'!
queue
	"A shared queue for process synchronization."

	^ queue! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:14:54.387'!
queue: aSharedQueue

	queue := aSharedQueue.! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:15:20.266'!
selector

	^ selector! !

!SignalConnectionConfiguration methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:15:43.137'!
selector: aSymbol

	selector := aSymbol.! !

Object subclass: #SignalConnectionPattern
	instanceVariableNames: 'mapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalConnectionPattern commentStamp: 'mt 6/24/2010 16:44:33.05' prior: 0!
Stores information about argument mapping and discarding as well as default arguments to be used during signal dispatching.!

!SignalConnectionPattern methodsFor: 'testing' stamp: 'mt 6/24/2010 17:33:43.284'!
hasIndices

	^ self indices notEmpty! !

!SignalConnectionPattern methodsFor: 'private' stamp: 'mt 6/24/2010 17:13:56.466'!
indices
	"Return an array with all reference indices. Discard defaults."
	
	| result nextIsDefault |
	result := OrderedCollection new.
	nextIsDefault := false.
	
	self mapping
		ifNotNil: [self mapping do: [:ea |
			nextIsDefault
				ifTrue: [nextIsDefault := false]
				ifFalse: [ea = #=
					ifTrue: [nextIsDefault := true]
					ifFalse: [result add: ea]]]].
		
	^ result! !

!SignalConnectionPattern methodsFor: 'testing' stamp: 'mt 6/24/2010 17:10:36.723'!
isEmpty

	^ self size = 0! !

!SignalConnectionPattern methodsFor: 'mapping' stamp: 'mt 6/24/2010 17:15:39.699'!
mapArguments: someArguments

	^ self mapArguments: someArguments useSender: nil! !

!SignalConnectionPattern methodsFor: 'mapping' stamp: 'mt 10/26/2010 16:19:17.37'!
mapArguments: someArguments useSender: aSender
	"Map index 0 to the sender."
	
	| result nextIsDefault offset |
	self mapping ifNil: [^ someArguments "Identity."].
	
	result := Array new: self size.
	nextIsDefault := false.
	offset := 0.
	
	self mapping withIndexDo: [:ea :index |
			nextIsDefault
				ifTrue: [nextIsDefault := false. result at: index - offset put: ea "Use default."]
				ifFalse: [ea = #=
					ifTrue: [nextIsDefault := true. offset := offset + 1 "Escape char found."]
					ifFalse: [
						ea = 0 "Send given sender as argument"
							ifTrue: [result at: index - offset put: aSender]
							ifFalse: [result at: index - offset put: (someArguments at: ea)]]]].
	
	^ result collect: [:each | each class == WeakArray
		ifTrue: [each first]
		ifFalse: [each]]! !

!SignalConnectionPattern methodsFor: 'accessing' stamp: 'mt 6/24/2010 16:57:51.358'!
mapping

	^ mapping! !

!SignalConnectionPattern methodsFor: 'accessing' stamp: 'mt 10/26/2010 16:18:18.273'!
mapping: anArray
	"#(1 2 3), #(1 2 =foobar), {#=. obj1. 2. 3}"

	mapping := anArray collect: [:each |
		({ByteString. ByteSymbol. SmallInteger} includes: each class)
			ifTrue: [each]
			ifFalse: [WeakArray with: each]]! !

!SignalConnectionPattern methodsFor: 'accessing' stamp: 'mt 6/24/2010 17:09:56.024'!
maxIndex

	^ self indices max! !

!SignalConnectionPattern methodsFor: 'accessing' stamp: 'mt 6/24/2010 17:23:58.127'!
minIndex

	^ self indices min! !

!SignalConnectionPattern methodsFor: 'accessing' stamp: 'mt 6/24/2010 17:11:47.229'!
numberOfNonDefaults

	^ self indices size! !

!SignalConnectionPattern methodsFor: 'accessing' stamp: 'mt 6/24/2010 17:07:37.003'!
size

	| nextIsDefault |
	nextIsDefault := false.
	
	^ self mapping
		ifNotNil: [self mapping count: [:ea |
			nextIsDefault
				ifTrue: [nextIsDefault := false]
				ifFalse: [ea = #=
					ifTrue: [nextIsDefault := true]
					ifFalse: [true]]]]
		ifNil: [0]! !

Object subclass: #SignalConsumer
	instanceVariableNames: 'value switch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/2/2010 21:43'!
switchState

	^ switch ifNil: [switch := #off]! !

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/2/2010 21:44'!
toggleSwitch

	switch := self switchState = #off
		ifTrue: [#on]
		ifFalse: [#off].! !

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/2/2010 21:43'!
value

	^ value! !

!SignalConsumer methodsFor: 'slots' stamp: 'mt 6/6/2010 09:01:01.648'!
value: aValue

	value := aValue.! !

!SignalConsumer methodsFor: 'signals' stamp: 'mt 6/13/2010 10:40:27.55'!
valueConsumed

	self emitAlways. "public signal"! !

SignalConsumer subclass: #SignalConsumerAuto
	instanceVariableNames: 'producer producerTwo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalConsumerAuto commentStamp: 'mt 6/6/2010 22:30:38.308' prior: 0!
I am a signal consumer that makes use of auto-connected signals.!

!SignalConsumerAuto methodsFor: 'initialize-release' stamp: 'mt 6/6/2010 23:06:23.56'!
initialize

	super initialize.
	producer := SignalProducer new.
	producerTwo := SignalProducer new.
	
	self createAutoSignalConnections.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 22:24:12.93'!
onProducerNewSignalProduced
	"sender: producer signal: #newSignalProduced"
	
	self toggleSwitch.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 23:06:13.035'!
onProducerTwoNewSignalProduced
	
	self toggleSwitch.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 22:25:06.762'!
onProducerValueChanged: aValue
	"sender: producer signal: #valueChanged:"
	
	self value: aValue.! !

!SignalConsumerAuto methodsFor: 'auto-connected actions' stamp: 'mt 6/6/2010 22:25:25.597'!
onProducerValueChanged: aValue and: anotherValue
	"sender: producer signal: #valueChanged:and:"
	
	self value: anotherValue.! !

!SignalConsumerAuto methodsFor: 'accessing' stamp: 'mt 6/6/2010 22:30:18.165'!
producer

	^ producer! !

!SignalConsumerAuto methodsFor: 'accessing' stamp: 'mt 6/6/2010 23:05:53.97'!
producerTwo

	^ producerTwo! !

Object subclass: #SignalKey
	instanceVariableNames: 'class selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalKey commentStamp: 'mt 6/14/2010 19:37:06.469' prior: 0!
I am a key used in the dictionary that holds all connections for a sending object in the connections repository.!

!SignalKey class methodsFor: 'instance creation' stamp: 'mt 6/14/2010 19:38:37.422'!
signal: aSignal sender: aSender

	^ self new
		theClass: aSender class;
		selector: aSignal.! !

!SignalKey methodsFor: 'comparing' stamp: 'mt 6/14/2010 19:49:03.511'!
= anotherKey

	^ anotherKey class = self class
		and: [{self theClass. self selector} = {anotherKey theClass. anotherKey selector}]! !

!SignalKey methodsFor: 'finalization' stamp: 'mt 6/14/2010 21:40:57.545'!
configureFinalize
	"What should happen, if I die?"

	(Smalltalk includesKey: #SignalMethodWrapper)
		ifTrue: [self toFinalizeSend: #checkAndConvertSignal: to: SignalMethodWrapper with: {self selector. self theClass}].! !

!SignalKey methodsFor: 'comparing' stamp: 'mt 6/14/2010 19:42:22.85'!
hash

	^ {self theClass. self selector} hash! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:40:11.679'!
selector

	^ selector! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:39:53.506'!
selector: aSelector

	selector := aSelector.! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:39:35.009'!
theClass

	^ class! !

!SignalKey methodsFor: 'accessing' stamp: 'mt 6/14/2010 19:39:42.439'!
theClass: aClass

	class := aClass.! !

Object subclass: #SignalProducer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/2/2010 21:39'!
newSignalProduced

	self emit.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/6/2010 12:20:50.329'!
newSignalProducedInBlock
	"Bad style."
	
	true
		ifTrue: [self emit]
		ifFalse: ["Do not emit."].! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 12:19:30.403'!
produceBadSignal

	self newSignalProducedInBlock.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:05:10.543'!
produceSignal

	self newSignalProduced.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:04:58.828'!
value: aValue

	self valueChanged: aValue.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:05:26.631'!
value: aValue and: anotherValue

	self valueChanged: aValue and: anotherValue.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/9/2010 08:47:27.027'!
valueChanged2: aValue
	"Public emittable signal."
	
	self emitAlways.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 10/4/2010 14:57:11.578'!
valueChanged: aValue

	self emit.! !

!SignalProducer methodsFor: 'signals' stamp: 'mt 6/2/2010 22:20'!
valueChanged: aValue and: anotherValue

	self emit.! !

!SignalProducer methodsFor: 'actions' stamp: 'mt 6/6/2010 09:34:05.91'!
valueInBlock: aValue

	[self valueChanged: aValue] value.! !

SignalProducer subclass: #SignalProducerEx
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalProducerEx commentStamp: 'mt 6/17/2010 15:12:46.235' prior: 0!
Used to test base class signal emitting.!

!SignalProducerEx methodsFor: 'actions' stamp: 'mt 6/17/2010 15:14:56.043'!
valueEx: newValue
	"Emit base class signal."

	self valueChanged: newValue.! !

Object subclass: #SignalSpy
	instanceVariableNames: 'queue verbose'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalSpy commentStamp: 'mt 6/3/2010 11:55' prior: 0!
Useful for testing signals.!

!SignalSpy class methodsFor: 'instance creation' stamp: 'mt 6/3/2010 12:24'!
onSender: aSender signal: aSignal

	^ self new
		addSender: aSender
		signal: aSignal! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/4/2010 13:00'!
addSender: aSender signal: aSignal

	[self
		connect: aSender
		signal: aSignal
		toSelector: aSignal] forkAndWait. "Avoid processing of signals in UI process."! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/21/2011 18:24:58.085'!
addSender: aSender signals: someSignals

	someSignals do: [:signal |
		self addSender: aSender signal: signal].! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/24/2011 09:22:42.368'!
doesNotUnderstand: aMessage

	self queue nextPut: {aMessage selector. aMessage arguments}.

	self verbose ifTrue: [
		Transcript
			cr;
			show: {aMessage selector. aMessage arguments}].! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/3/2010 11:58'!
queue

	^ queue ifNil: [queue := SharedQueue new]! !

!SignalSpy methodsFor: 'spying' stamp: 'mt 6/3/2010 12:00'!
respondsTo: aSelector

	^ true! !

!SignalSpy methodsFor: 'accessing' stamp: 'mt 6/21/2011 18:27:33.016'!
verbose
	"Print-out on transcript for each received signal."
	
	^ verbose ifNil: [false]! !

!SignalSpy methodsFor: 'accessing' stamp: 'mt 6/21/2011 18:25:39.756'!
verbose: aBoolean

	verbose := aBoolean.! !

!SignalSpy methodsFor: 'waiting' stamp: 'mt 6/3/2010 12:07'!
waitForNextSignal

	^ self queue next! !

!SignalSpy methodsFor: 'waiting' stamp: 'mt 6/23/2010 16:11:35.105'!
waitForNextSignalSatisfying: aBlock

	| signal |
	[signal := self queue next. aBlock value: signal]
		whileFalse.
	^ signal! !

!SignalSpy methodsFor: 'waiting' stamp: 'mt 6/3/2010 12:21'!
waitForSignal: aSignal

	| signal |
	[signal notNil and: [signal first = aSignal]]
		whileFalse: [signal := self queue next].
		
	^ signal! !

Object subclass: #SignalTranscriptModel
	instanceVariableNames: 'lastUpdate lastEntry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalTranscriptModel commentStamp: 'mt 6/22/2011 10:02:01.744' prior: 0!
A helper class to catch transcript outputs.

Usage:

| stm |
stm := SignalTranscriptModel new.
stm register.

Transcript show: #foobar.

self assert: #appendEntry equals: stm lastUpdate.
self assert: #foobar equals: stm lastEntry.

stm unregister.!

!SignalTranscriptModel methodsFor: 'accessing' stamp: 'mt 6/22/2011 09:58:55.61'!
lastEntry

	^ lastEntry! !

!SignalTranscriptModel methodsFor: 'accessing' stamp: 'mt 6/22/2011 09:58:49.773'!
lastUpdate

	^ lastUpdate! !

!SignalTranscriptModel methodsFor: 'dependents' stamp: 'mt 6/22/2011 09:59:41.079'!
register

	Transcript addDependent: self.! !

!SignalTranscriptModel methodsFor: 'dependents' stamp: 'mt 6/22/2011 09:59:48.357'!
unregister

	Transcript removeDependent: self.! !

!SignalTranscriptModel methodsFor: 'updating' stamp: 'mt 6/22/2011 09:58:39.337'!
update: aSymbol

	lastUpdate := aSymbol.

	aSymbol = #appendEntry
		ifTrue: [lastEntry := Transcript contents].! !

TestCase subclass: #SignalKeyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalKeyTest methodsFor: 'tests' stamp: 'mt 6/14/2010 20:02:34.653'!
test01KeyInDictionary

	| dict |
	dict := Dictionary new.
	dict at: (SignalKey signal: #valueChanged: sender: SignalProducer new) put: #foobar.
	
	self assert: #foobar equals: (dict at: (SignalKey signal: #valueChanged: sender: SignalProducer new)).! !

TestCase subclass: #SignalPatternTest
	instanceVariableNames: 'pattern'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalPatternTest methodsFor: 'running' stamp: 'mt 6/24/2010 16:42:58.214'!
setUp

	pattern := SignalConnectionPattern new.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 17:24:15.52'!
test01SimplePattern

	pattern mapping: #(5 4 3 2 1).
	
	self
		assert: 5 equals: pattern size;
		deny: pattern isEmpty;
		assert: 5 equals: pattern maxIndex;
		assert: 1 equals: pattern minIndex.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 16:47:14.141'!
test02EmptyPattern

	self assert: SignalConnectionPattern new isEmpty.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 16:56:41.05'!
test03PatternWithDefaults

	pattern mapping: #(1 2 =foobar).
	
	self
		assert: 3 equals: pattern size;
		assert: 2 equals: pattern numberOfNonDefaults.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 16:53:54.483'!
test04SimpleMapping

	pattern mapping: #(3 1 2).
	
	self assert: {#foobar. 'dog'. 5} equals: (pattern mapArguments: {'dog'. 5. #foobar}).! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 17:33:00.689'!
test05MappingWithDefaults

	pattern mapping: #(2 1 =foobar).
	
	self
		assert: pattern hasIndices;
		assert: {5. 'dog'. #foobar} equals: (pattern mapArguments: {'dog'. 5}).! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 17:12:39.356'!
test06MaxIndexWithDefaults

	pattern mapping: #(2 1 =foobar).
	
	self assert: 2 equals: pattern maxIndex.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 17:24:28.065'!
test07MinIndexWithDefaults

	pattern mapping: #(2 1 =foobar).
	
	self assert: 1 equals: pattern minIndex.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 17:35:24.642'!
test08DefaultsOnly

	pattern mapping: #(=dog =foobar).
	
	self
		deny: pattern hasIndices;
		assert: 0 equals: pattern numberOfNonDefaults.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 6/24/2010 17:47:59.921'!
test09DefaultObjects

	pattern mapping: {#=. Morph new color: Color yellow}.
	
	self
		assert: Color yellow equals: (pattern mapArguments: {}) first color.! !

!SignalPatternTest methodsFor: 'tests' stamp: 'mt 10/26/2010 16:32:13.691'!
test10weakMapping

	pattern mapping: {#=. Morph new color: Color yellow}.
	
	Smalltalk garbageCollect.
	
	self assert: (pattern mapArguments: {}) first isNil.! !

TestCase subclass: #SignalTest
	instanceVariableNames: 'producer consumer process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Signals-Tests'!

!SignalTest commentStamp: 'mt 3/25/2011 11:22:50.279' prior: 0!
Warning: Running those tests may clear the local signal connection repository. All applications that make use of them need to be restarted.!

!SignalTest methodsFor: 'running' stamp: 'mt 6/2/2010 22:42'!
setUp

	consumer := SignalConsumer new.
	producer := SignalProducer new.! !

!SignalTest methodsFor: 'running' stamp: 'mt 6/3/2010 08:19'!
tearDown

	super tearDown.
	(process notNil and: [process isTerminated not]) ifTrue: [process terminate].! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/2/2010 22:00'!
test01ConnectDisconnect

	self assert: producer signalConnections isEmpty.
	consumer connect: producer signal: #valueChanged: toSelector: #value:.
	self assert: 1 equals: producer signalConnections size.
	
	self
		assert: (producer disconnectSignal: #valueChanged:);
		assert: producer signalConnections isEmpty;
		deny: (producer disconnectSignal: #valueChanged:).
		
	self connect: producer signal: #valueChanged: to: consumer selector: #value:.
	self assert: 1 equals: producer signalConnections size.

	self
		assert: (producer disconnectSignal: #valueChanged: from: consumer);
		assert: producer signalConnections isEmpty;
		deny: (producer disconnectSignal: #valueChanged: from: consumer).
		
	self connect: producer signal: #valueChanged: to: consumer selector: #value:.
	
	self
		assert: (producer disconnectSignal: #valueChanged: from: consumer selector: #value:);
		assert: producer signalConnections isEmpty;
		deny: (producer disconnectSignal: #valueChanged: from: consumer selector: #value:).
		
	self connect: producer signal: #valueChanged: to: consumer selector: #value:.
	self 
		deny: (producer disconnectSignal: #shouldNotExist);
		deny: producer signalConnections isEmpty.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/2/2010 22:00'!
test02ConnectError

	self
		should: [consumer connect: producer signal: #shouldNotExist toSelector: #toggleSwitch] raise: Error;
		should: [consumer connect: producer signal: #valueChanged: toSelector: #shouldNotExist] raise: Error;
		should: [self connect: producer signal: #shouldNotExist to: consumer selector: #toggleSwitch] raise: Error;
		should: [self connect: producer signal: #valueChanged: to: consumer selector: #shouldNotExist] raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:01.776'!
test03Signaling

	consumer connect: producer signal: #valueChanged: toSelector: #value:.
	consumer connect: producer signal: #newSignalProduced toSelector: #toggleSwitch.
	
	self
		assert: consumer value isNil;
		assert: #off equals: consumer switchState.	
	producer value: 'foobar'.
	producer produceSignal.
	self
		assert: consumer value = 'foobar';
		assert: #on equals: consumer switchState.	
	
	producer disconnectAllSignals.
	producer value: 'blubb'.
	self
		assert: consumer value = 'foobar';
		assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:06.376'!
test04Patterns

	consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch pattern: #().
	
	self assert:#off equals: consumer switchState.	
	producer value: 'foobar'.
	self assert:#on equals: consumer switchState.	! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:13.531'!
test05PatternsReorder

	consumer connect: producer signal: #valueChanged:and: toSelector: #value: pattern: #(2).
	
	self assert: consumer value isNil.
	producer value: 'foobar' and: 'blubb'.
	self assert: 'blubb' equals: consumer value.	! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:22.928'!
test06DoubleConnection

	consumer connect: producer signal: #valueChanged: toSelector: #value:.
	consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch pattern: #().
	
	self
		assert: consumer value isNil;
		assert: #off equals: consumer switchState.	
	
	producer value: 'foobar'.
	
	self
		assert: consumer value = 'foobar';
		assert: #on equals: consumer switchState.	! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/4/2010 13:39'!
test07PatternError

	self
		should: [consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch pattern: #(1)] raise: Error;
		should: [consumer connect: producer signal: #valueChanged: toSelector: #value: pattern: #(2)] raise: Error;
		should: [consumer connect: producer signal: #valueChanged:with: toSelector: #value: pattern: #(2 1)] raise: Error;
		should: [consumer connect: producer signal: #valueChanged: toSelector: #value: pattern: #()] raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:34.539'!
test08QueuedConnection

	| queue result |
	queue := SharedQueue new.
	process := [queue next value. result := consumer value] forkAt: 60.
	
	consumer connect: producer signal: #valueChanged: toSelector: #value: queue: queue.
	producer value: #QueuedConnectionsAreCool.
	
	Processor yield. "Let the result be set."
	
	self assert: #QueuedConnectionsAreCool equals: result.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:48.546'!
test09QueuedConnectionToUI

	consumer
		connect: producer
		signal: #valueChanged:
		toSelector: #value:
		queue: WorldState deferredUIMessages.
		
	producer value: #QueuedConnectionsAreCool.
	
	self assert: nil equals: consumer value.
	World doOneCycle. "Let the result be set. Do it manually here because we ARE already in the UI process."
	self assert: #QueuedConnectionsAreCool equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:54.378'!
test10BlockingConnection

	| result |
	consumer
		connect: producer
		signal: #valueChanged:
		toSelector: #value:
		queue: WorldState deferredUIMessages
		blocking: true.
		
	process := [producer value: #QueuedConnectionsAreCool. result := 'blubb'] forkAt: 60.
	Processor yield.
	
	self assert: nil equals: result.
	
	World doOneCycle. "Deliver the signal."
	Processor yield. "Let the result be set."
	
	self assert: 'blubb' equals: result.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:06:58.852'!
test11SignalSpy

	| spy |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	producer value: #DontSpyOnMe.	
	self assert: {#valueChanged:. {#DontSpyOnMe}} equals: spy waitForNextSignal.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:04.379'!
test12SignalSpyConcurrent

	| spy |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	process := [producer value: #DontSpyOnMe] fork.
	self assert: {#valueChanged:. {#DontSpyOnMe}} equals: spy waitForNextSignal.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:15.917'!
test13SignalSpySpecificSignal

	| spy |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	process := [
		producer produceSignal. "will be discarded"
		producer value: #DontSpyOnMe] fork.
	self assert: {#valueChanged:. {#DontSpyOnMe}} equals: (spy waitForSignal: #valueChanged:).! !

!SignalTest methodsFor: 'tests' stamp: 'mt 9/28/2012 11:41:30.065'!
test14DefaultUIProcessing
	"Check whether default connections out of the ui process will only be processed in the ui process."
	
	self assert: UIManager default uiProcess equals: Processor activeProcess.
	
	consumer
		connect: producer
		signal: #valueChanged:
		toSelector: #value:.
		
	process := [producer value: #QueuedConnectionsAreCool] forkAt: 60.
	Processor yield.
	
	self assert: nil equals: consumer value.
	
	World doOneCycle. "Deliver the signal."
	
	self assert: #QueuedConnectionsAreCool equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/4/2010 13:28'!
test15ArgumentCountError

	self
		should: [consumer connect: producer signal: #newSignalProduced toSelector: #value:]
		raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:30.717'!
test16ArgumentAutoTruncate

	consumer connect: producer signal: #valueChanged: toSelector: #toggleSwitch.
	
	self assert: #off equals: consumer switchState.
	producer value: #foobar.
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:07:35.016'!
test17ConnectSignalToSignal

	producer connect: producer signal: #valueChanged: toSelector: #newSignalProduced.
	consumer connect: producer signal: #newSignalProduced toSelector: #toggleSwitch.
	
	self assert: #off equals: consumer switchState.
	producer value: #foobar.
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 09:34:17.452'!
test18SignalFromOutsideError
	"There should be an error if another class tries to emit a signal from the outside."
	
	self
		should: [producer valueChanged: #foobar] raise: Error;
		should: [[producer valueChanged: #foobar] value] raise: Error;
		shouldnt: [producer valueInBlock: #foobar] raise: Error.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 11:46:34.493'!
test19UseSenderAsArgument
	
	consumer connect: producer signal: #newSignalProduced toSelector: #value: pattern: #(0).
	producer produceSignal.
	
	self assert: producer equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 12:19:57.244'!
test20BadSignal
	
	consumer connect: producer signal: #newSignalProducedInBlock toSelector: #toggleSwitch.
	producer produceBadSignal.
	
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 22:34:30.515'!
test21AutoConnections
	
	| c |
	c := SignalConsumerAuto new.
	
	self
		assert: #off equals: c switchState;
		assert: nil equals: c value.
		
	c producer produceSignal; value: #foobar.
	
	self
		assert: #on equals: c switchState;
		assert: #foobar equals: c value.
		
	c producer value: #foobar and: #blubb.
	
	self
		assert: #on equals: c switchState;
		assert: #blubb equals: c value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/6/2010 23:07:24.806'!
test22AutoConnectionsTwo
	
	| c |
	c := SignalConsumerAuto new.
	
	self assert: #off equals: c switchState.
		
	c producerTwo produceSignal.
	
	self assert: #on equals: c switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/9/2010 08:57:42.069'!
test23EmitPublicSignal

	consumer connect: producer signal: #valueChanged2: toSelector: #value:.
	
	self assert: nil equals: consumer value.
	producer valueChanged2: #foobar.
	self assert: #foobar equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/24/2010 16:34:52.66'!
test24ConnectSignalToSignal2

	consumer connect: producer signal: #valueChanged: toSelector: #valueConsumed. "#valueConsumed is a public signal."
	consumer connect: consumer signal: #valueConsumed toSelector: #toggleSwitch.
	
	self assert: #off equals: consumer switchState.
	producer value: #foobar.
	self assert: #on equals: consumer switchState.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/14/2010 21:19:42.768'!
test25WeakRepositoryFinalization

	SignalConnectionsRepository reset.

	SignalConsumer new
		connect: SignalProducer new
		signal: #valueChanged:
		toSelector: #valueConsumed.
	
	"Let's finalize."
	Smalltalk garbageCollect.
	Processor yield.

	self assert: SignalConnectionsRepository instance isEmpty.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/17/2010 15:14:25.87'!
test26BaseClassSignals

	| producerEx |
	producerEx := SignalProducerEx new.
	
	consumer connect: producerEx signal: #valueChanged: toSelector: #value:.
	
	self assert: nil equals: consumer value.
	producerEx value: #foobar.
	self assert: #foobar equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/17/2010 15:40:50.535'!
test27BaseClassSignalsEx

	| producerEx |
	producerEx := SignalProducerEx new.
	
	consumer connect: producerEx signal: #valueChanged: toSelector: #value:.
	
	self assert: nil equals: consumer value.
	producerEx valueEx: #foobar.
	self assert: #foobar equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 10/26/2010 15:31:32.434'!
test28DefaultArgsInConnectionSymbol

	consumer
		connect: producer
		signal: #newSignalProduced
		toSelector: #value:
		pattern: #(=foobar).

	producer produceSignal.

	self assert: #foobar equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 10/26/2010 15:31:44.854'!
test29DefaultArgsInConnectionInteger

	consumer
		connect: producer
		signal: #newSignalProduced
		toSelector: #value:
		pattern: #(=5).

	producer produceSignal.

	self assert: 5 equals: consumer value.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 10/26/2010 15:31:07.482'!
test30DefaultArgsInConnectionObject

	consumer
		connect: producer
		signal: #newSignalProduced
		toSelector: #value:
		pattern: {#=. Morph new color: Color yellow}.

	producer produceSignal.

	self assert: Color yellow equals: consumer value color.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 3/25/2011 11:27:00.455'!
test31DefaultArgsAndGarbageCollection

	| weakArray morph signalConsumer signalProducer |
	morph := Morph new.
	signalConsumer := SignalConsumer new.
	signalProducer := SignalProducer new.
	
	weakArray := WeakArray
		with: morph
		with: signalConsumer
		with: signalProducer.
	
	self deny: (SignalConnectionsRepository instance includesKey: signalProducer).

	signalConsumer
		connect: signalProducer
		signal: #newSignalProduced
		toSelector: #value:
		pattern: {#=. morph}.

	self
		assert: (SignalConnectionsRepository instance includesKey: signalProducer);
		assert: ({morph. signalConsumer. signalProducer} hasEqualElements: weakArray).
	
	morph := nil.
	signalConsumer := nil.
	signalProducer := nil.

	SignalConnectionsRepository instance finalizeLater.
	Smalltalk garbageCollect.
	Processor yield. "Let weak array finalization process run."

	self
		deny: (SignalConnectionsRepository instance includesKey: signalProducer);
		assert: ({nil. nil. nil} hasEqualElements: weakArray copy).! !

!SignalTest methodsFor: 'tests' stamp: 'mt 3/25/2011 11:27:13.679'!
test32DefaultArgsAndGarbageCollection2

	| weakArray signalConsumer signalProducer |
	signalConsumer := SignalConsumer new.
	signalProducer := SignalProducer new.
	
	weakArray := WeakArray
		with: signalConsumer
		with: signalProducer.
	
	self deny: (SignalConnectionsRepository instance includesKey: signalProducer).

	signalConsumer
		connect: signalProducer
		signal: #newSignalProduced
		toSelector: #value:
		pattern: {#=. signalProducer}.

	self
		assert: (SignalConnectionsRepository instance includesKey: signalProducer);
		assert: ({signalConsumer. signalProducer} hasEqualElements: weakArray).
	
	signalConsumer := nil.
	signalProducer := nil.

	SignalConnectionsRepository instance finalizeLater.
	Smalltalk garbageCollect.
	Processor yield. "Let weak array finalization process run."

	self
		deny: (SignalConnectionsRepository instance includesKey: signalProducer);
		assert: ({nil. nil} hasEqualElements: weakArray copy).! !

!SignalTest methodsFor: 'tests' stamp: 'mt 3/25/2011 11:27:20.347'!
test33DefaultArgsAndGarbageCollection3

	| weakArray signalConsumer signalProducer |
	signalConsumer := SignalConsumer new.
	signalProducer := SignalProducer new.
	
	weakArray := WeakArray
		with: signalConsumer
		with: signalProducer.
	
	self deny: (SignalConnectionsRepository instance includesKey: signalProducer).

	signalConsumer
		connect: signalProducer
		signal: #newSignalProduced
		toSelector: #value:
		pattern: {#=. signalConsumer}.

	self
		assert: (SignalConnectionsRepository instance includesKey: signalProducer);
		assert: ({signalConsumer. signalProducer} hasEqualElements: weakArray).
	
	signalConsumer := nil.
	signalProducer := nil.

	SignalConnectionsRepository instance finalizeLater.
	Smalltalk garbageCollect.
	Processor yield. "Let weak array finalization process run."

	self
		deny: (SignalConnectionsRepository instance includesKey: signalProducer);
		assert: ({nil. nil} hasEqualElements: weakArray copy).! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/22/2011 09:56:10.673'!
test34SignalSpyMultiple

	| spy |
	spy := SignalSpy new.
	spy addSender: producer signals: #(valueChanged: valueChanged:and:).

	producer
		value: #DontSpyOnMe;
		value: #LetMeAlone and: #GetAHike.

	self
		assert: {#valueChanged:. {#DontSpyOnMe}} equals: spy waitForNextSignal;
		assert: {#valueChanged:and:. {#LetMeAlone. #GetAHike}} equals: spy waitForNextSignal.! !

!SignalTest methodsFor: 'tests' stamp: 'mt 6/22/2011 10:05:13.425'!
test35SignalSpyVerboseMode

	| spy stm |
	spy := SignalSpy onSender: producer signal: #valueChanged:.
	spy verbose: true.
	
	stm := SignalTranscriptModel new register.
	
	producer value: #DontSpyOnMe.	
	
	self
		assert: #appendEntry equals: stm lastUpdate;
		assert: String cr, '#(#valueChanged: #(#DontSpyOnMe))' equals: stm lastEntry.
	
	stm unregister.! !

!MorphicUIManager methodsFor: '*signals-core' stamp: 'mt 9/30/2012 14:11:30.476'!
uiProcess
	"Compatibility for Pharo."

	(Smalltalk includesKey: #Project)
		ifTrue: [^ (Smalltalk at: #Project) uiProcess].

	self flag: #pharo.
	(self class classVarNames includes: #UIProcess)
		ifTrue: [^ self class classPool at: #UIProcess].! !

WeakIdentityKeyDictionary subclass: #SignalConnectionsRepository
	instanceVariableNames: 'mutex lastFinalizeTime finalizeTimeOut'
	classVariableNames: 'Repository'
	poolDictionaries: ''
	category: 'Signals-Core'!

!SignalConnectionsRepository commentStamp: 'mt 6/2/2010 22:43' prior: 0!
I store all active signal-selector connections with weak data structures.!

!SignalConnectionsRepository class methodsFor: 'testing' stamp: 'mt 6/14/2010 21:37:53.926'!
checkSignal: aSignal inClass: aClass
	"Check whether there are connections for the signal in the class."
	
	self critical: [
		self instance associationsDo: [:assoc |
			assoc key ifNotNil: [:sender | (sender class = aClass and: [
					assoc value keys anySatisfy: [:signalKey |
						signalKey selector = aSignal]]) ifTrue: [^ true]]]].

	^ false! !

!SignalConnectionsRepository class methodsFor: 'instance' stamp: 'mt 6/3/2010 08:50'!
critical: aBlock

	^ self instance critical: aBlock! !

!SignalConnectionsRepository class methodsFor: 'instance' stamp: 'mt 6/14/2010 20:43:52.302'!
instance

	^ Repository ifNil: [Repository := self new. WeakArray addWeakDependent: Repository. Repository]! !

!SignalConnectionsRepository class methodsFor: 'instance' stamp: 'mt 6/14/2010 20:47:18.789'!
reset

	WeakArray removeWeakDependent: Repository.
	Repository := nil.! !

!SignalConnectionsRepository methodsFor: 'critical' stamp: 'mt 6/3/2010 08:46'!
critical: aBlock

	mutex ifNil: [mutex := Mutex new].
	^ mutex critical: aBlock! !

!SignalConnectionsRepository methodsFor: 'finalization' stamp: 'mt 3/25/2011 11:26:33.707'!
finalizeLater

	lastFinalizeTime := nil.! !

!SignalConnectionsRepository methodsFor: 'accessing' stamp: 'mt 3/25/2011 11:20:17.33'!
finalizeTimeOut
	"Time between cleanup."
	
	^ finalizeTimeOut ifNil: [5000 "ms"]! !

!SignalConnectionsRepository methodsFor: 'accessing' stamp: 'mt 3/25/2011 11:20:28.221'!
finalizeTimeOut: milliseconds
	
	finalizeTimeOut := milliseconds.! !

!SignalConnectionsRepository methodsFor: 'finalization' stamp: 'mt 3/28/2011 10:02:31.184'!
finalizeValues

	| interval |
	interval := lastFinalizeTime
		ifNil: [self finalizeTimeOut]
		ifNotNil: [:t | (t to: DateAndTime now) duration asMilliSeconds].
	
	interval < 0
		ifTrue: [interval := self finalizeTimeOut].
	
	interval < self finalizeTimeOut
		ifTrue: [^ self].
		
	self critical: [super finalizeValues].
	
	lastFinalizeTime := DateAndTime now.! !

!SignalConnectionsRepository methodsFor: 'accessing' stamp: 'mt 3/28/2011 10:08:03.155'!
mutex

	^ mutex ifNil: [mutex := Mutex new]! !
